package cz.cvut.kbss.bookstore.dao;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * Implements operations of the {@link GenericDao} interface.
 *
 * @param <T> Entity type
 * @param <ID> Entity id type
 */
public abstract class BaseDao<T, ID> implements GenericDao<T, ID> {

    private final Class<T> type;

    @PersistenceContext
    protected EntityManager em;

    protected BaseDao(Class<T> type) {
        this.type = type;
    }

    @Override
    public List<T> findAll() {
        return em.createQuery("SELECT e FROM " + type.getName() + " e", type).getResultList();
    }

    @Override
    public T find(ID id) {
        return em.find(type, id);
    }

    @Override
    public Long getCount() {
        return em.createQuery("SELECT count(e) FROM " + type.getName() + " e", Long.class).getSingleResult();
    }

    @Override
    public void remove(T instance) {
        em.remove(instance);
    }

    @Override
    public void persist(T instance) {
        em.persist(instance);
    }

    @Override
    public void update(T instance) {
        em.merge(instance);
    }

    @Override
    public T loadById(ID id) {
        return em.getReference(type, id);
    }
}
