package cz.cvut.kbss.bookstore.service;

import cz.cvut.kbss.bookstore.dao.GenericDao;
import cz.cvut.kbss.bookstore.dao.UserDao;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.cache.NullUserCache;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

/**
 *
 * @author kidney
 */
public class AuthenticationService extends AbstractUserDetailsAuthenticationProvider {
    
    private static final Logger LOG = LoggerFactory.getLogger(AuthenticationService.class);
    
    private UserDao userDAO;

    private TransactionTemplate transactionTemplate;

    public AuthenticationService() {
        this.setUserCache(new NullUserCache());
    }

    public void setUserDAO(UserDao userDAO) {
        this.userDAO = userDAO;
    }

    public void setTransactionTemplate(TransactionTemplate transactionTemplate) {
        this.transactionTemplate = transactionTemplate;
    }

    @Override
    protected void additionalAuthenticationChecks(UserDetails ud, UsernamePasswordAuthenticationToken upat) throws AuthenticationException {
        // Do nothing
    }

    @Override
    protected UserDetails retrieveUser(final String username, final UsernamePasswordAuthenticationToken upat) throws AuthenticationException {
        //only public methods can be marked as transactional
        return (UserDetails) transactionTemplate.execute(new TransactionCallback() {

            @Override
            public Object doInTransaction(TransactionStatus status) {
                try {
                    UserDetails ud = null;

                    cz.cvut.kbss.bookstore.bo.User u;
                    try {
                        u = userDAO.findByUsername(username);                        
                    } catch (IllegalArgumentException erdaex) {
                        throw new BadCredentialsException("User not found.");
                    }
                    String password = (String) upat.getCredentials();
                    if (u == null || !u.hasPassword(password)) {
                        AuthenticationException e = new BadCredentialsException("Invalid credentials.");
                        throw e;
                    } else {
                        List<GrantedAuthority> auths = new ArrayList<GrantedAuthority>();
                        auths.add(new SimpleGrantedAuthority("ROLE_USER"));
                        if(u.isAdmin()){
                            auths.add(new SimpleGrantedAuthority("ROLE_ADMIN"));
                        }
                        ud = new User(u.getUserName(), u.getPassword(), auths);
                    }
                    return ud;
                } catch(AuthenticationException e){
                    status.setRollbackOnly();
                    throw e;
                }catch (Exception e) {
                    LOG.error("Error occured during retrieveUser call", e);
                    status.setRollbackOnly();
                    throw new RuntimeException(e);
                }
            }
        });
    }
}
