package cz.cvut.kbss.bookstore.service;

import cz.cvut.kbss.bookstore.bo.Book;
import cz.cvut.kbss.bookstore.dao.BookDao;
import cz.cvut.kbss.bookstore.dao.GenericDao;
import cz.cvut.kbss.bookstore.dao.UserDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * As can be seen here, interfaces are not strictly necessary for beans.
 * 
 * If a concrete class without interface should be used as bean, Spring configuration has to use cglib proxies instead of JDK based.
 * See the tx:annotation-driven element in applicationContext.xml.
 * The same effect is acheived using @EnableAspectJAutoProxy(proxyTargetClass = true) in the ServiceConfig class.
 * @author ledvima1
 */
@Service
public class BookService extends AbstractDataAccessService<Book, Long> {

    @Autowired
    private BookDao bookDao;

    @Autowired
    private UserDao userDao;

    @Override
    protected GenericDao<Book, Long> getPrimaryDao() {
        return bookDao;
    }

    @Transactional(readOnly = true)
    public List<Book> getUsersBooks(Long userId) {
        return bookDao.getBooksByUser(userDao.loadById(userId));
    }

    public Long addBook(String title, Long owner) {
        final Book newBook = new Book();
        newBook.setTitle(title);
        newBook.setOwner(userDao.loadById(owner));

        persist(newBook);
        return newBook.getId();
    }
}
