package cz.cvut.kbss.bookstore.service;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Base interface for all services which abstract the persistence layer.
 *
 * @param <T> Entity type
 * @param <ID> Identifier type
 */
public interface DataAccessService<T, ID> {

    @Transactional(readOnly = true)
    T find(ID id);

    @Transactional(readOnly = true)
    List<T> findAll();

    @Transactional
    void persist(T instance);

    @Transactional
    void remove(T instance);

    @Transactional
    void removeById(ID id);

    @Transactional
    void update(T instance);

}
