package cz.cvut.kbss.bookstore.service;

import cz.cvut.kbss.bookstore.bo.User;
import cz.cvut.kbss.bookstore.dao.GenericDao;
import cz.cvut.kbss.bookstore.dao.UserDao;
import cz.cvut.kbss.bookstore.dto.UserDto;
import cz.cvut.kbss.bookstore.helper.DtoTransformerHelper;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service("userService")
public class UserService extends AbstractDataAccessService<User, Long> {

    @Autowired
    private UserDao userDao;

    @Override
    protected GenericDao<User, Long> getPrimaryDao() {
        return userDao;
    }

    @Transactional
    public Long addUser(String userName, String password, int age) {
        User newUser = new User();
        newUser.setAge(age);
        newUser.setPassword(password);
        newUser.setUserName(userName);

        persist(newUser);
        return newUser.getId();
    }

    public UserDto findAsDto(Long id) {
        final User u = find(id);
        if (u == null) {
            return null;
        }
        return new UserDto(u.getId(), u.getUserName(), u.getAge(), DtoTransformerHelper.getIdentifiers(u.getBooks()));
    }

    public List<UserDto> findAllAsDto() {
        
        return findAll().stream()
                .map(u -> new UserDto(u.getId(), u.getUserName(), u.getAge(), DtoTransformerHelper.getIdentifiers(u.getBooks())))
                .collect(Collectors.toList());      
    }
}
