package cz.cvut.kbss.bookstore.service;

import cz.cvut.kbss.bookstore.bo.Book;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

import static org.junit.Assert.assertEquals;

public class BookServiceTest extends AbstractServiceTest {

    @Autowired
    private BookService bookService;

    @Autowired
    private UserService userService;

    @Test
    public void testAddAndRetrieveBook() {

        Long userId = addUser();

        String title = "Bob a Bobek, kralici z klobouku";

        Long bookId = bookService.addBook(title, userId);
        List<Book> books = bookService.getUsersBooks(userId);
        assertEquals(1, books.size());

        Book book = books.get(0);

        assertEquals(title, book.getTitle());
        assertEquals(userId, book.getOwner().getId());
        assertEquals(bookId, book.getId());
    }

    @Test
    public void testAddAndRemoveBook() {
        Long userId = addUser();
        
        String title = "Bob a Bobek, kralici z klobouku";
        Long bookId = bookService.addBook(title, userId);
        assertEquals(1, bookService.findAll().size());
        bookService.removeById(bookId);
        assertEquals(0, bookService.findAll().size());
    }
    
    @Test
    public void testBookDeletedWhenUserRemoved(){
        Long userId = addUser();
        
        String title = "Bob a Bobek, kralici z klobouku";
        bookService.addBook(title, userId);
        assertEquals(1, bookService.findAll().size());
        
        userService.removeById(userId);    
        assertEquals(0, bookService.findAll().size());
        
    }

    private long addUser() {
        String userName = "UserName" + System.currentTimeMillis();
        String passwd = "passwd" + System.currentTimeMillis();
        int age = 18;

        return userService.addUser(userName, passwd, age);
    }
}
